﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using MySql.Data;
using MySql.Data.MySqlClient;


/// <summary>
/// Descripción breve de DataBaseClass
/// </summary>
public class DataBaseClass
{
    private static MySqlConnection connection;

    #region CONNECTION
    public DataBaseClass()
	{
        connection = Connection();
	}
    public static MySqlConnection Connection()
    {
        string cad = "Database=students;Data Source=localhost;User Id=root;Password=root";
       return new MySqlConnection(cad);
    }

    public void Connect()
    {
        connection.Open();
    }
  
    public void Disconnect()
    {
        connection.Close();
    }
    #endregion

    #region SELECTS

    public Course FindCourse(string coursename) 
    {
        Course course = null;
        try
        {
            string sel = "select * from course where coursename=?coursename";
            MySqlCommand com = new MySqlCommand(sel, connection);
            MySqlParameter p = new MySqlParameter("?coursename", MySqlDbType.String);
            p.Value = coursename;
            com.Parameters.Add(p);
            Connect();
            MySqlDataReader reader = com.ExecuteReader();
            
          
            if (reader.HasRows)
            {
                reader.Read();
                course = new Course();
                course.setCourseid(reader.GetInt16(0));
                course.setCoursename(reader.GetString(1));
            }
                Disconnect();
        }
          catch (Exception ex)
        {
            Disconnect();
            throw ex;
            
        }
        return course;
        
    }

    public DataTable FindAllCourses()
    {
        DataTable dt=new DataTable();
        try
        {
            string sel = "select * from course";
            MySqlDataAdapter menuda = new MySqlDataAdapter(sel, connection);
            Connect();
            menuda.Fill(dt);
            Disconnect();
        }
        catch (Exception ex)
        {
            Disconnect();
            throw ex;
            
        }
        return dt;

    }

    public Student_info FindStudent(string studentname,DateTime birth)
    {
        Student_info student = null;
        try
        {
            string sel = "select * from student_info where studentname=?studentname and birth=?birth";
            MySqlCommand com = new MySqlCommand(sel, connection);
            MySqlParameter p = new MySqlParameter("?studentname", MySqlDbType.String);
            p.Value = studentname;
            com.Parameters.Add(p);
            p = new MySqlParameter("?birth", MySqlDbType.Datetime);
            p.Value = birth;
            com.Parameters.Add(p);
            Connect();
            MySqlDataReader reader = com.ExecuteReader();
            
            if (reader.HasRows)
            {
                reader.Read();
                student = new Student_info();
                student.setStudentid(reader.GetInt16(0));
                student.setStudentname(reader.GetString(1));
                student.setBirth(reader.GetDateTime(2));

            }
            Disconnect();
        }
        catch(Exception ex)
        {
            Disconnect();
            throw ex;
        }
        return student;

    }
    public Week_eval FindWeek(int weeknr,int studentid)
    {
        Week_eval week = null;
        try
        {
            string sel = "select * from week_eval where weeknr=?weeknr and studentid=?studentid";
            MySqlCommand com = new MySqlCommand(sel, connection);
            MySqlParameter p = new MySqlParameter("?weeknr", MySqlDbType.Int16);
            p.Value = weeknr;
            com.Parameters.Add(p);
            p = new MySqlParameter("?studentid", MySqlDbType.Int16);
            p.Value = studentid;
            com.Parameters.Add(p);
            Connect();
            MySqlDataReader reader = com.ExecuteReader();

            if (reader.HasRows)
            {
                reader.Read();
                week = new Week_eval();
                week.setIdweek(reader.GetInt16(0));
                week.setStudentid(reader.GetInt16(1));
                week.setWeekmark(reader.GetInt16(2));
                week.setWeeknr(reader.GetInt16(3));
                week.setCourseid(reader.GetInt16(4));
                week.setComments(reader.GetString(5));

            }
            Disconnect();
        }
        catch (Exception ex)
        {
            Disconnect();
            throw ex;
        }
        return week;

    }

    public DataTable FindWeeks(int weeknr)
    {
        DataTable weeks = new DataTable();
        try
        {
            string sel = "select * from week_eval where weeknr=?weeknr order by studentid";
            MySqlCommand com = new MySqlCommand(sel, connection);
            MySqlParameter p = new MySqlParameter("?weeknr", MySqlDbType.Int16);
            p.Value = weeknr;
            com.Parameters.Add(p);
            MySqlDataAdapter da = new MySqlDataAdapter(com);
            Connect();
            da.Fill(weeks);
            Disconnect();
        }
        catch (Exception ex)
        {
            Disconnect();
            throw ex;
        }
        return weeks;

    }

    public DataTable FindStudentsInCourse(Course course)
    {
        DataTable dt= new DataTable();
        DataTable DtWeeks=new DataTable();
        DataRow row;
        int studentid,indexweek;
        try
        {
            string sel = "select student_info.studentid,studentname,birth from student_info,course_activity where course_activity.courseid=?courseid and course_activity.studentid=student_info.studentid";
            MySqlCommand com = new MySqlCommand(sel, connection);
            MySqlParameter p = new MySqlParameter("?courseid", MySqlDbType.String);
            p.Value = course.getCourseid();
            com.Parameters.Add(p);
            MySqlDataAdapter da = new MySqlDataAdapter(com);
      
            Connect();
            da.Fill(dt);
            Disconnect();
            int weeks = FindNumWeeks();
         //weeks columns
                for (int i = 1; i <= weeks; i++)//for each week column
                {
                    dt.Columns.Add(new DataColumn(/*"Week " +*/ i.ToString()));//add week column
                    DtWeeks = FindWeeks(i);
                    if (DtWeeks.Rows.Count!=0)//if this week have marks
                    {
                        indexweek = 0;
                    int index = 0;
                    while(index<dt.Rows.Count && indexweek<DtWeeks.Rows.Count)//while students and weeks 
                    {
                       row= dt.Rows[index];//row=student row
                       if (DtWeeks.Rows[indexweek][1].ToString().CompareTo(row[0].ToString())==0)//student id from student_info=studentid from week_eval
                        {
                            row[/*"Week " +*/ i.ToString()] = DtWeeks.Rows[indexweek][2];//grid row updated with mark
                            indexweek++;
                        }
                        index++;
                    }
                    }
                }       
        }
        catch (Exception ex)
        {
            Disconnect();
            throw ex;
        }
        return dt;

    }

    public int FindNumWeeks()
    {
         DataTable dt = new DataTable();
        try
        {
            string sel = "select max(weeknr) as weeks from week_eval";
            MySqlDataAdapter da = new MySqlDataAdapter(sel,connection);
            Connect();
            da.Fill(dt);
            Disconnect();            
        }
        catch (Exception ex)
        {
            Disconnect();
            throw ex;
        }
        DataRow dr = dt.Rows[0];
        return Convert.ToInt16(dr["weeks"].ToString());
    }

    #endregion

    #region INSERTS

    public void InsertCourse(Course course)
    {
        try
        {
            string ins = "insert into course (coursename) values(?coursename)";
            MySqlCommand com = new MySqlCommand(ins, connection);
            MySqlParameter p = new MySqlParameter("?coursename", MySqlDbType.String);
            p.Value = course.getCoursename();
            com.Parameters.Add(p);
            Connect();
            com.ExecuteNonQuery();
            Disconnect();
        }
        catch
        {
            Disconnect();
        }
    }
    public void InsertStudent(Student_info student)
    {
        try
        {
            string ins = "insert into student_info (studentname,birth) values(?studentname,?birth)";
            MySqlCommand com = new MySqlCommand(ins, connection);
            MySqlParameter p = new MySqlParameter("?studentname", MySqlDbType.String);
            p.Value = student.getStudentname();
            com.Parameters.Add(p);
            p = new MySqlParameter("?birth", MySqlDbType.Datetime);
            p.Value = student.getBirth();
            com.Parameters.Add(p);
            
            Connect();
            com.ExecuteNonQuery();
            Disconnect();
        }
        catch
        {
            Disconnect();
        }
    }

    public void InsertCourseActivity(Course_activity ca)
    {
        try
        {
            string ins = "insert into course_activity (courseid,studentid) values(?courseid,?studentid)";
            MySqlCommand com = new MySqlCommand(ins, connection);
            MySqlParameter p = new MySqlParameter("?courseid", MySqlDbType.Int16);
            p.Value = ca.getCourseid();
            com.Parameters.Add(p);
            p = new MySqlParameter("?studentid", MySqlDbType.Int16);
            p.Value = ca.getStudentid();
            com.Parameters.Add(p);
            Connect();
            com.ExecuteNonQuery();
            Disconnect();
        }
        catch
        {
            Disconnect();
        }
    }

    public void InsertStudentCourseActivity(Student_courseactivity sca)
    {
        try
        {
           string ins = "insert into student_courseactivity (courseid,studentid,class) values(?courseid,?studentid,?class)";
            MySqlCommand com = new MySqlCommand(ins, connection);
            MySqlParameter p = new MySqlParameter("?courseid", MySqlDbType.Int16);
            p.Value = sca.getCourseid();
            com.Parameters.Add(p);
            p = new MySqlParameter("?studentid", MySqlDbType.Int16);
            p.Value = sca.getStudentid();
            com.Parameters.Add(p);
            p = new MySqlParameter("?class", MySqlDbType.String);
            p.Value = sca.getClassroom();
            com.Parameters.Add(p);
            Connect();
            com.ExecuteNonQuery();
            Disconnect();
        }
        catch
        {
            Disconnect();
        }
    }

    public void InsertWeek(Week_eval week)
    {
        try
        {
            string ins = "insert into week_eval (studentid,weekmark,weeknr,courseid,comment) values(?studentid,?weekmark,?weeknr,?courseid,?comment)";
            MySqlCommand com = new MySqlCommand(ins, connection);
            MySqlParameter p = new MySqlParameter("?studentid", MySqlDbType.Int16);
            p.Value = week.getStudentid();
            com.Parameters.Add(p);
            p = new MySqlParameter("?weekmark", MySqlDbType.Int16);
            p.Value = week.getWeekmark();
            com.Parameters.Add(p);
            p = new MySqlParameter("?weeknr", MySqlDbType.Int16);
            p.Value = week.getWeeknr();
            com.Parameters.Add(p);
            p = new MySqlParameter("?courseid", MySqlDbType.Int16);
            p.Value = week.getCourseid();
            com.Parameters.Add(p);
            p = new MySqlParameter("?comment", MySqlDbType.String);
            p.Value = week.getComments();
            com.Parameters.Add(p);
            Connect();
            com.ExecuteNonQuery();
            Disconnect();
        }
        catch (Exception ex)
        {
            Disconnect();
            throw ex;
        }
    }

    #endregion

    #region UPDATES

    public void UpdateWeek(Week_eval week)
    {
        try
        {
            string ins = "update week_eval set weekmark=?weekmark,comment=?comment where idweek=?idweek";
            MySqlCommand com = new MySqlCommand(ins, connection);
            MySqlParameter p = new MySqlParameter("?weekmark", MySqlDbType.Int16);
            p.Value = week.getWeekmark();
            com.Parameters.Add(p);
            p = new MySqlParameter("?comment", MySqlDbType.String);
            p.Value = week.getComments();
            com.Parameters.Add(p);
            p = new MySqlParameter("?idweek", MySqlDbType.Int16);
            p.Value =week.getIdweek();
            com.Parameters.Add(p);
            Connect();
            com.ExecuteNonQuery();
            Disconnect();
        }
        catch
        {
            Disconnect();
        }
    }

    #endregion
}
