<?

class CsvHandler  {

  var $csvPath;
  var $separator = ";";
  var $errors    = true;
  var $warnings  = true;

  // CONSTRUCTOR AND ACCESSORS
  // =========================

  // initializes instance
  function CsvHandler($cPath, $sep=";") {
    $this->csvPath   = $cPath;
    $this->separator = $sep;
    if (!file_exists($cPath)) $this->writeCsv("");
  }

  // setter for $warnings
  function showWarnings($bool) {
    $this->$warnings = $bool;
  }

  // setter for $warnings
  function showErrors($bool) {
    $this->$errors = $bool;
  }

  // 'PRIVATE' FUNCTIONS
  // ===================

  // writes content in $csvPath
  function writeCsv($content) {
    $fp = @fopen($this->csvPath, "w+") or $this->showError("Error: could not write CSV file");
    fwrite($fp, $content);
    fclose($fp);
  }

  // handle warnings and errors
  function showWarning($war) {
    if ($this->warnings) echo $war;
  }

  function showError($err) {
    if ($this->errors) die ($err);
  }

  // 'PUBLIC' FUNCTIONS
  // ==================

  // READERS

  // returns csv in 1d array
  function toArray() {
    return @file($this->csvPath) or $this->show_error("Error: CSV file not found");
  }

  // returns csv in 2d array
  function to2darray() {
    $rows = @file($this->csvPath) or $this->show_error("Error: CSV file not found");
    $out;
    foreach ($rows as $row) {
      $out[] = explode($this->separator, $row);
    }
    return $out;
  }

  // returns csv in table
  function toTable($id="") {
    $rows = @file($this->csvPath) or $this->show_error("Error: CSV file not found");
    $out = "<table id='".$id."'>\n";
    foreach ($rows as $row) {
      $out .= "  <tr>\n";
      $cells = explode($this->separator, $row);
      foreach ($cells as $cell) {
        $out .= "    <td>".$cell."</td>\n";
      }
      $out .= "  </tr>\n";
    }
    $out .= "</table>\n";
    return $out;
  }

  // WRITERS

  // writes csv from 1d array
  function fillFromArray($arr) {
    $content = implode("\n", $arr)."\n";
    $content = ereg_replace("\n+","\n",$content);
    $this->writeCsv($content);
  }

  // writes csv from 2d array
  function fillFrom2darray($arr) {
    $tmp;
    foreach ($el as $arr) {
      $tmp[] = implode($this->separator, $el);
    }
    $this->fillFromArray($tmp);
  }

  // checks if row exists
  function rowExists($val, $colNr=0) {
    $rows = @file($this->csvPath) or $this->showError("Error: could not read CSV file");
    foreach ($rows as $row) {
      $cells = explode($this->separator, $row);
      if ($cells[$colNr]==$val) return true;
    }
    return false;
  }

  // MODIFIERS

  // removes row at specified index
  function removeRow($index) {
    $rows = @file($this->csvPath) or $this->showError("Error: could not read CSV file");
    if ($index>=count($rows)) {
      $this->showWarning("Warning: row to remove does not exist");
      return;
    }
    array_splice($rows, $index, 1);
    $this->fillFromArray($rows);
  }

  // adds row at specified index
  function addRow($val, $index=-1) {
    $row = (is_array($val)?implode($this->separator, $val):$val);
    $rows = @file($this->csvPath) or $this->showError("Error: could not read CSV file");
    if (isset($rows[0]) && count(explode($this->separator, $rows[0]))!=count(explode($this->separator,$row))) {
      $this->showWarning("Warning: could not add row - field count mismatch (make sure you use the '".$this->separator."' separator ");
      return;
    }
    if ($index==-1) $rows[] .= $row;
    if ($index!=-1) {
      $rows[] = array_slice($rows, 0, $index-1) + $row + array_slice($rows, $index);
    }
    $this->fillFromArray($rows);
  }

}

?>
